// Service.java - provide services to Gluon Cloud
// Copyright 2015, Anderson Software Group, Inc.
// Gail and Paul Anderson
package com.asgteach.bpcloud.service;

import com.asgteach.bpcloud.model.BPData;
import com.gluonhq.cloudlink.client.data.DataClient;
import com.gluonhq.cloudlink.client.data.DataClientBuilder;
import com.gluonhq.cloudlink.client.data.SyncFlag;
import com.gluonhq.cloudlink.client.user.User;
import com.gluonhq.cloudlink.client.user.UserClient;
import com.gluonhq.connect.ConnectState;
import com.gluonhq.connect.GluonObservableList;
import com.gluonhq.connect.provider.DataProvider;
import javafx.beans.Observable;
import javafx.beans.property.ListProperty;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleListProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.collections.FXCollections;
import javax.annotation.PostConstruct;

/**
 *
 * @author gail
 */
public class Service {

    // include property extractors so we can listen for updates
    private final ListProperty<BPData> bpList
            = new SimpleListProperty<>(FXCollections.observableArrayList((BPData param) -> new Observable[]{
        param.systolicProperty(),
        param.diastolicProperty(),
        param.pulseProperty()
    }));

    private DataClient dataClient;

    private UserClient userClient;

    private final ObjectProperty<User> user = new SimpleObjectProperty<>();

    @PostConstruct
    public void postConstruct() {
        user.addListener((obs, ov, nv) -> {
            if (nv != null) {
                System.out.println(nv.getKey()
                        + " for "
                        + nv.getName());
            }
        });
        userClient = new UserClient();
        // Note: the authentication will fail if you don't have your application
        // correctly configured through the Gluon Dashboard at https://gluon.io
        dataClient = DataClientBuilder.create()
                .authenticateWith(userClient)
                .build();

        user.bind(userClient.authenticatedUserProperty());
        System.out.println("got data client");
    }

    // Call this after authentication
    // Store user's data in the cloud based on the user "nickname" and
    // "networkID". It should be unique for each user.
    // Requires that the user authenticates via the same social network site
    // to access the data from other devices.
    private void getData(User auth_user) {
        System.out.println("Get Data called.");
        GluonObservableList<BPData> gluonBPData
                = DataProvider.<BPData>retrieveList(dataClient.createListDataReader(
                        auth_user.getNick() + auth_user.getNetworkId(), BPData.class,
                        SyncFlag.LIST_READ_THROUGH, SyncFlag.LIST_WRITE_THROUGH,
                        SyncFlag.OBJECT_READ_THROUGH, SyncFlag.OBJECT_WRITE_THROUGH));

        gluonBPData.stateProperty().addListener((obs, ov, nv) -> {
            if (ConnectState.SUCCEEDED.equals(nv)) {
                System.out.println("Service-getData with authenticated user : connection succeeded.");
                bpList.set(gluonBPData);
            }
        });
    }

    public void retrieveReadings() {
        userClient.authenticate(this::getData);
    }

    public BPData addReading(BPData data) {
        bpList.get().add(data);
        return data;
    }

    public void removeReading(BPData data) {
        bpList.get().remove(data);
    }

    public ListProperty<BPData> bpListProperty() {
        return bpList;
    }

    public void clear() {
        System.out.println("clearing " + bpList.get().size() + " items.");
        bpList.get().clear();
    }

    public ObjectProperty<User> userProperty() {
        return user;
    }

    public User getUser() {
        return user.get();
    }

}
