/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.asgteach.bpcloud.views;

import com.asgteach.bpcloud.model.BPData;
import com.gluonhq.charm.glisten.control.CharmListCell;
import com.gluonhq.charm.glisten.control.ListTile;
import com.gluonhq.charm.glisten.visual.MaterialDesignIcon;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.function.Consumer;
import javafx.beans.value.ChangeListener;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.HBox;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;

/**
 *
 * @author gail
 */
public class BPDataCell extends CharmListCell<BPData> {

    private final ChangeListener<Number> bpChangeListener = (obs, ov, nv) -> update();

    private final ListTile listTile = new ListTile();
    private final Circle circle = new Circle(20, Color.WHITE);
    private final Label label = new Label("22");

    private static final DateTimeFormatter ftime
            = DateTimeFormatter.ofLocalizedTime(FormatStyle.SHORT);
    private static final DateTimeFormatter fdate
            = DateTimeFormatter.ofLocalizedDate(FormatStyle.MEDIUM);
    private BPData currentItem;

    public BPDataCell(Consumer<BPData> edit, Consumer<BPData> remove) {

        final Button buttonEdit = MaterialDesignIcon.EDIT.button(e -> edit.accept(currentItem));
        final Button buttonDel = MaterialDesignIcon.DELETE.button(e -> remove.accept(currentItem));
        circle.setStroke(Color.DODGERBLUE);
        circle.setStrokeWidth(2);
        StackPane sp = new StackPane(circle, label);
        listTile.setPrimaryGraphic(sp);
        listTile.setSecondaryGraphic(new HBox(buttonEdit, buttonDel));

        setText(null);
        update();
    }

    private void update() {
        if (currentItem != null) {
            System.out.println("update for currrentItem " + currentItem);
            listTile.textProperty().setAll(currentItem.getSystolic() + "/"
                    + currentItem.getDiastolic() + " mmHg" + "\t"
                    + currentItem.getPulse() + " bpm",
                    currentItem.getDate().format(fdate)
                    + "\t" + currentItem.getDate().format(ftime),
                    "\t");
            int systolic = currentItem.getSystolic();

            label.setText(currentItem.getDate().getDayOfMonth() + "");

            // Grab the node that holds the primary text
            // so we can style it based on the systolic reading
            Node node = listTile.lookup(".primary-text");
            if (systolic <= 130) {
                node.setStyle("-fx-text-fill: green;");
            } else if (systolic > 130 && systolic <= 140) {
                node.setStyle("-fx-text-fill: orange;");
            } else {
                node.setStyle("-fx-text-fill: red;");
            }
        } else {
            listTile.textProperty().clear();
        }

    }

    @Override
    public void updateItem(BPData item, boolean empty) {
        super.updateItem(item, empty);
        updateCurrentItem(item);
        if (item != null && !empty) {
            update();
            setGraphic(listTile);

        } else {
            setGraphic(null);
        }
    }

    private void updateCurrentItem(BPData data) {
        if (currentItem == null || !currentItem.equals(data)) {
            if (currentItem != null) {
                currentItem.systolicProperty().removeListener(bpChangeListener);
                currentItem.diastolicProperty().removeListener(bpChangeListener);
                currentItem.pulseProperty().removeListener(bpChangeListener);
            }

            currentItem = data;

            if (currentItem != null) {
                currentItem.systolicProperty().addListener(bpChangeListener);
                currentItem.diastolicProperty().addListener(bpChangeListener);
                currentItem.pulseProperty().addListener(bpChangeListener);
            }
        }
    }

}
